
/*
**
**  $VER: CloneMethods.c 1.1 (19.3.97)
**  datatypes.library/Examples/CloneMethods
**
**  Shows the usage of CopyDTMethods and CopyDTTriggerMethods
**  (should only be used in within datatypes classes)
**
**  Written 1996/97 by Roland 'Gizzy' Mainz
**
*/

/* amiga includes */
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <dos/rdargs.h>
#include <dos/dosextens.h>
#include <datatypes/datatypes.h>
#include <datatypes/datatypesclass.h>
#include <datatypes/animationclass.h>

/* amiga prototypes */
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/datatypes_protos.h>

/* amiga pragmas */
#include <pragmas/exec_pragmas.h>
#include <pragmas/dos_pragmas.h>
#include <pragmas/datatypes_pragmas.h>
#include <pragmas/alib_pragmas.h> /* tagcall stubs */

/* ansi includes */
#include <string.h>


/*****************************************************************************/

extern struct Library *SysBase,
                      *DOSBase;

struct Library *DataTypesBase;

/*****************************************************************************/

/* local prototypes */
static void              PrintSupportedMethods( ULONG * );
static void              CloneMethodsTest( void );
static void              PrintSupportedTriggerMethods( struct DTMethod * );
static ULONG            *VerboseCopyDTMethods( ULONG *, ULONG *, ULONG * );
static struct DTMethod  *VerboseCopyDTTriggerMethods( struct DTMethod *, struct DTMethod *, struct DTMethod * );

/*****************************************************************************/


int main( void )
{
    /* Open the libraries */
    if( DataTypesBase = OpenLibrary( "datatypes.library", 45UL ) )
    {
      CloneMethodsTest();

      CloseLibrary( DataTypesBase );
    }

    return( 0 );
}


static
void CloneMethodsTest( void )
{
    /* CopyDTMethods test */
    {
      ULONG methods[] =
      {
        OM_NEW,
        OM_DISPOSE,
        OM_GET,
        OM_SET,
        OM_UPDATE,
        OM_NOTIFY,
        DTM_NEWMEMBER,
        ~0UL
      };

      ULONG includemethods[] =
      {
        DTM_TRIGGER,
        ~0UL
      };

      ULONG excludemethods[] =
      {
        DTM_NEWMEMBER,
        DTM_DISPOSEMEMBER,
        ~0UL
      };

      ULONG *clonedmethods;

      if( clonedmethods = VerboseCopyDTMethods( methods, includemethods, excludemethods ) )
      {
        FreeDTMethods( clonedmethods );
      }
    }

    /* CopyDTTriggerMethods test */
    {
      struct DTMethod triggermethods[] =
      {
        "Play",      "PLAY",        STM_PLAY,
        "Pause",     "PAUSE",       STM_PAUSE,
        "Stop",      "STOP",        STM_STOP,
        "Play from", "PLAYFROM",    (STM_PLAY | STMD_ULONG),
        "Stop at",   "STOPAT",      (STM_STOP | STMD_ULONG),
        NULL,        NULL,          0UL
      };

      struct DTMethod includetriggermethods[] =
      {
        "Locate frame", "LOCATEFRAME", (STM_LOCATE | STMD_ULONG),
        NULL,           NULL,          0UL
      };

      struct DTMethod excludetriggermethods[] =
      {
        "Pause",     "PAUSE",       STM_PAUSE,
        NULL,        NULL,          0UL
      };

      struct DTMethod *cloneddtm;

      if( cloneddtm = VerboseCopyDTTriggerMethods( triggermethods, includetriggermethods, excludetriggermethods ) )
      {
        FreeDTMethods( cloneddtm );
      }
    }
}



static
ULONG *VerboseCopyDTMethods( ULONG *methods, ULONG *include, ULONG *exclude )
{
    Printf( "Original methods array looks like this:\n" );
    PrintSupportedMethods( methods );

    Printf( "Following will be included (add) in the methods array:\n" );
    PrintSupportedMethods( include );

    Printf( "Following will be excluded (remove) from the methods array:\n" );
    PrintSupportedMethods( exclude );

    methods = CopyDTMethods( methods, include, exclude );

    Printf( "Result looks like this:\n" );
    PrintSupportedMethods( methods );

    return( methods );
}


static 
struct DTMethod *VerboseCopyDTTriggerMethods( struct DTMethod *methods, struct DTMethod *include, struct DTMethod *exclude )
{
    Printf( "Original trigger methods array looks like this:\n" );
    PrintSupportedTriggerMethods( methods );

    Printf( "Following will be included (add) in the trigger methods array:\n" );
    PrintSupportedTriggerMethods( include );

    Printf( "Following will be excluded (remove) from the trigger methods array:\n" );
    PrintSupportedTriggerMethods( exclude );

    methods = CopyDTTriggerMethods( methods, include, exclude );

    Printf( "Result trigger method array looks like this:\n" );
    PrintSupportedTriggerMethods( methods );

    return( methods );
}


static
void PrintSupportedMethods( ULONG *methods )
{
    if( methods )
    {
      Printf( "methods array contains:\n" );

      while( *methods != ~0UL )
      {
        switch( *methods )
        {
/* rootclass */
          case OM_NEW:              Printf( "OM_NEW\n" );               break;
          case OM_DISPOSE:          Printf( "OM_DISPOSE\n" );           break;
          case OM_SET:              Printf( "OM_SET\n" );               break;
          case OM_GET:              Printf( "OM_GET\n" );               break;
          case OM_ADDTAIL:          Printf( "OM_ADDTAIL\n" );           break;
          case OM_REMOVE:           Printf( "OM_REMOVE\n" );            break;
          case OM_NOTIFY:           Printf( "OM_NOTIFY\n" );            break;
          case OM_UPDATE:           Printf( "OM_UPDATE\n" );            break;
          case OM_ADDMEMBER:        Printf( "OM_ADDMEMBER\n" );         break;
          case OM_REMMEMBER:        Printf( "OM_REMMEMBER\n" );         break;
/* gadgetclass */
          case GM_HITTEST:          Printf( "GM_HITTEST\n" );           break;
          case GM_RENDER:           Printf( "GM_RENDER\n" );            break;
          case GM_GOACTIVE:         Printf( "GM_GOACTIVE\n" );          break;
          case GM_HANDLEINPUT:      Printf( "GM_HANDLEINPUT\n" );       break;
          case GM_GOINACTIVE:       Printf( "GM_GOINACTIVE\n" );        break;
          case GM_HELPTEST:         Printf( "GM_HELPTEST\n" );          break;
          case GM_LAYOUT:           Printf( "GM_LAYOUT\n" );            break;
/* datatypesclass */
          case DTM_FRAMEBOX:        Printf( "DTM_FRAMEBOX\n" );         break;
          case DTM_PROCLAYOUT:      Printf( "DTM_PROCLAYOUT\n" );       break;
          case DTM_ASYNCLAYOUT:     Printf( "DTM_ASYNCLAYOUT\n" );      break;
          case DTM_REMOVEDTOBJECT:  Printf( "DTM_REMOVEDTOBJECT\n" );   break;
          case DTM_SELECT:          Printf( "DTM_SELECT\n" );           break;
          case DTM_CLEARSELECTED:   Printf( "DTM_CLEARSELECTED\n" );    break;
          case DTM_COPY:            Printf( "DTM_COPY\n" );             break;
          case DTM_PRINT:           Printf( "DTM_PRINT\n" );            break;
          case DTM_ABORTPRINT:      Printf( "DTM_ABORTPRINT\n" );       break;
          case DTM_NEWMEMBER:       Printf( "DTM_NEWMEMBER\n" );        break;
          case DTM_DISPOSEMEMBER:   Printf( "DTM_DISPOSEMEMBER\n" );    break;
          case DTM_GOTO:            Printf( "DTM_GOTO\n" );             break;
          case DTM_TRIGGER:         Printf( "DTM_TRIGGER\n" );          break;
          case DTM_OBTAINDRAWINFO:  Printf( "DTM_OBTAINDRAWINFO\n" );   break;
          case DTM_DRAW:            Printf( "DTM_DRAW\n" );             break;
          case DTM_RELEASEDRAWINFO: Printf( "DTM_RELEASEDRAWINFO\n" );  break;
          case DTM_WRITE:           Printf( "DTM_WRITE\n" );            break;
/* animation.datatype */
          case ADTM_LOADFRAME:      Printf( "ADTM_LOADFRAME\n" );       break;
          case ADTM_UNLOADFRAME:    Printf( "ADTM_UNLOADFRAME\n" );     break;
          case ADTM_START:          Printf( "ADTM_START\n" );           break;
          case ADTM_PAUSE:          Printf( "ADTM_PAUSE\n" );           break;
          case ADTM_STOP:           Printf( "ADTM_STOP\n" );            break;
          case ADTM_LOCATE:         Printf( "ADTM_LOCATE\n" );          break;
          default:                  Printf( "<%lx>\n", *methods );      break;
        }

        methods++;
      }
    }
    else
    {
      Printf( "no methods\n" );
    }
}


static
void PrintSupportedTriggerMethods( struct DTMethod *dtm )
{
    if( dtm )
    {
      Printf( "Follwoing trigger methods are defined:\n" );

      while( (dtm -> dtm_Method) != STM_DONE )
      {
        Printf( "Label: '%10.10s' ",   (dtm -> dtm_Label) );
        Printf( "Command: '%10.10s' ", (dtm -> dtm_Command) );
        Printf( "Method: %10.10lu ",   (dtm -> dtm_Method) );

        switch( GetDTTriggerMethodDataFlags( (dtm -> dtm_Method)) )
        {
          case STMD_VOID:    Printf( "(VOID)"     ); break;
          case STMD_ULONG:   Printf( "(ULONG)"    ); break;
          case STMD_STRPTR:  Printf( "(STRPTR)"   ); break;
          case STMD_TAGLIST: Printf( "(TAGLIST)"  ); break;
          default:           Printf( "--UNKOWN--" ); break;
        }

        Printf( "\n" );

        dtm++;
      }
    }
    else
    {
      Printf( "No trigger methods defined\n" );
    }
}


