
/*
**
**  $VER: DoDT.c 1.2 (27.3.97)
**  datatypes.library/Examples/DoDT
**
**  Shows how to examine a file using datatypes.library,
**  find out which tools supports this particular data type
**  and meybe launch them.
**
**  Written 1996/97 by Roland 'Gizzy' Mainz
**
*/

/* amiga includes */
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <dos/rdargs.h>
#include <dos/dosextens.h>
#include <dos/dostags.h>
#include <datatypes/datatypes.h>
#include <datatypes/datatypesclass.h>
#include <libraries/iffparse.h>

/* amiga prototypes */
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/iffparse_protos.h>
#include <clib/datatypes_protos.h>

/* amiga pragmas */
#include <pragmas/exec_pragmas.h>
#include <pragmas/dos_pragmas.h>
#include <pragmas/iffparse_pragmas.h>
#include <pragmas/datatypes_pragmas.h>
#include <pragmas/alib_pragmas.h> /* tagcall stubs */

/* ansi includes */
#include <string.h>


/*****************************************************************************/

extern struct Library *SysBase,
                      *DOSBase;

struct Library *DataTypesBase,
               *IFFParseBase;

/*****************************************************************************/

#define TEMPLATE "NAMES/M/A,MISC/S,INFO/S,BROWSE/S,EDIT/S,PRINT/S,MAIL/S,ALL/S,LAUNCH/S,ASYNCLAUNCH/S"
#define OPT_NAMES        (0)
#define OPT_MISC         (1)
#define OPT_INFO         (2)
#define OPT_BROWSE       (3)
#define OPT_EDIT         (4)
#define OPT_PRINT        (5)
#define OPT_MAIL         (6)
#define OPT_ALL          (7)
#define OPT_LAUNCH       (8)
#define OPT_ASYNCLAUNCH  (9)
#define OPT_MAX         (10)

/*****************************************************************************/

int main( void )
{
    if( (SysBase -> lib_Version) < 39UL )
    {
      Printf( "requires 3.x or beyond to run\n" );
    }
    else
    {
      ULONG          options[ OPT_MAX ];
      struct RDArgs *rdargs;

      /* Clear the option array */
      memset( options, 0, sizeof( options ) );

      /* Parse the arguments */
      if( rdargs = ReadArgs( TEMPLATE, (LONG *)options, NULL ) )
      {
        /* Open the libraries */
        if( DataTypesBase = OpenLibrary( "datatypes.library", 45UL ) )
        {
          if( IFFParseBase = OpenLibrary( "iffparse.library", 39UL ) )
          {
            struct TagItem  toolargs[ 2 ];
            STRPTR         *names;

            toolargs[ 0 ] . ti_Tag  = NP_Synchronous;
            toolargs[ 0 ] . ti_Data = (ULONG)(options[ OPT_ASYNCLAUNCH ] == 0UL);
            toolargs[ 1 ] . ti_Tag  = TAG_DONE;
            toolargs[ 1 ] . ti_Data = 0UL;

            /* Get a pointer to the name array */
            names = (STRPTR *)options[ OPT_NAMES ];

            /* Step through the name array */
            while( *names )
            {
              BPTR lock;

              /* Lock the current name */
              if( lock = Lock( *names, ACCESS_READ ) )
              {
                struct DataType *dtn;

                /* Determine the DataType of the file */
                if( dtn = ObtainDataTypeA( DTST_FILE, (APTR)lock, NULL ) )
                {
                  struct DataTypeHeader *dth;
                  UBYTE                  buffer[ 6 ];

                  dth = dtn -> dtn_Header;

                  /* Print file name */
                  Printf( ">> Information on: '%s'\t<<\n", *names );

                  /* Show (and launch, on request) all tools, type MISC */
                  if( options[ OPT_MISC ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_MISC, TAG_DONE ) )
                    {
                      Printf( "misc \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools, type INFO */
                  if( options[ OPT_INFO ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_INFO, TAG_DONE ) )
                    {
                      Printf( "info \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools, type BROWSE */
                  if( options[ OPT_BROWSE ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_BROWSE, TAG_DONE ) )
                    {
                      Printf( "browse \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools, type EDIT */
                  if( options[ OPT_EDIT ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_EDIT, TAG_DONE ) )
                    {
                      Printf( "edit \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools, type PRINT */
                  if( options[ OPT_PRINT ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_PRINT, TAG_DONE ) )
                    {
                      Printf( "print \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools, type MAIL */
                  if( options[ OPT_MAIL ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNode( (struct List *)tn, TOOLA_Which, TW_MAIL, TAG_DONE ) )
                    {
                      Printf( "mail \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  /* Show (and launch, on request) all tools */
                  if( options[ OPT_ALL ] )
                  {
                    struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

                    while( tn = FindToolNodeA( (struct List *)tn, NULL ) )
                    {
                      Printf( "supported by: \"%s\"\n", (tn -> tn_Tool . tn_Program) );

                      if( options[ OPT_LAUNCH ] )
                      {
                        LaunchToolA( (&(tn -> tn_Tool)), *names, toolargs );
                      }
                    }
                  }

                  Printf( "   Description: '%s'\n",   (dth -> dth_Name) );
                  Printf( "     Base Name: '%s'\n",   (dth -> dth_BaseName) );
                  Printf( "          Type: '%s'\n",   GetDTString( (ULONG)(dth -> dth_Flags & DTF_TYPE_MASK) + DTMSG_TYPE_OFFSET) );
                  Printf( "         Group: '%s'\n",   GetDTString( (dth -> dth_GroupID) ) );
                  Printf( "            ID: '%s'\n\n", IDtoStr( (dth -> dth_ID), buffer ) );

                  /* Release the DataType */
                  ReleaseDataType( dtn );
                }
                else
                {
                  /* ObtainDataTypeA failed */
                  PrintFault( IoErr(), *names );
                }

                UnLock( lock );
              }
              else
              {
                /* Lock failed */
                PrintFault( IoErr(), *names );
              }

              /* Get the next name */
              names++;
            }

            CloseLibrary( IFFParseBase );
          }
          else
          {
            /* Couldn't open iffparse.library, Version 39 or higher */
            Printf( "Couldn't open iffparse.library V39\n" );
          }

          CloseLibrary( DataTypesBase );
        }
        else
        {
          /* Couldn't open datatypes.library, Version 45 or higher */
          Printf( "Couldn't open datatypes.library V45\n" );
        }

        /* Free the allocated memory after ReadArgs */
        FreeArgs( rdargs );
      }
      else
      {
        /* An error occured */
        PrintFault( IoErr(), "DoDT" );
      }
    }
    
    return( RETURN_OK );
}



