
/*
**
**  $VER: DTDescrList.c 1.1 (19.3.97)
**  datatypes.library/Examples/DTDescrList
**
**  Dumps out the list of installed datatypes or
**  find the datatypes which matches a given file.
**
**  Written 1996/97 by Roland 'Gizzy' Mainz
**
*/

/* amiga includes */
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <dos/rdargs.h>
#include <dos/dosextens.h>
#include <datatypes/datatypes.h>
#include <datatypes/datatypesclass.h>

/* amiga prototypes */
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/datatypes_protos.h>

/* amiga pragmas */
#include <pragmas/dos_pragmas.h>
#include <pragmas/exec_pragmas.h>
#include <pragmas/datatypes_pragmas.h>
#include <pragmas/alib_pragmas.h> /* tagcall stubs */

/* ansi includes */
#include <string.h>

/*****************************************************************************/

extern struct Library *SysBase,
                      *DOSBase;

/*****************************************************************************/

struct Library *DataTypesBase;

/*****************************************************************************/

#define TEMPLATE "NAME,"                         \
                 "GID_SYSTEM=SYSTEM/S,"          \
                 "GID_TEXT=TEXT/S,"              \
                 "GID_DOCUMENT=DOCUMENT/S,"      \
                 "GID_SOUND=SOUND/S,"            \
                 "GID_INSTRUMENT=INSTRUMENT/S,"  \
                 "GID_MUSIC=MUSIC/S,"            \
                 "GID_PICTURE=PICTURE/S,"        \
                 "GID_ANIMATION=ANIMATION/S,"    \
                 "GID_MOVIE=MOVIE/S,"            \
                 "TOOLS/S"

#define OPT_NAME        (0)
#define OPT_SYSTEM      (1)
#define OPT_TEXT        (2)
#define OPT_DOCUMENT    (3)
#define OPT_SOUND       (4)
#define OPT_INSTRUMENT  (5)
#define OPT_MUSIC       (6)
#define OPT_PICTURE     (7)
#define OPT_ANIMATION   (8)
#define OPT_MOVIE       (9)
#define OPT_TOOLS      (10)
#define OPT_MAX        (11)


/*****************************************************************************/

/* local prototypes */
static void PrintDTToolList( struct DataType * );

/*****************************************************************************/

void main( void )
{
    struct DataTypeHeader *dth;
    struct DataType       *dtn;

    LONG                   options[ OPT_MAX ];
    struct RDArgs         *rdargs;

    if( (SysBase -> lib_Version) < 39UL )
    {
      Printf( "requires 3.x or beyond to run\n" );
    }
    else
    {
      /* Clear the option array */
      memset( options, 0, sizeof( options ) );

      /* Parse the arguments */
      if( rdargs = ReadArgs( TEMPLATE, options, NULL ) )
      {
        ULONG groupid = 0UL;
        ULONG i;
        const
        ULONG gid_table[] =
        {
          GID_SYSTEM,
          GID_TEXT,
          GID_DOCUMENT,
          GID_SOUND,
          GID_INSTRUMENT,
          GID_MUSIC,
          GID_PICTURE,
          GID_ANIMATION,
          GID_MOVIE
        };

        for( i = OPT_SYSTEM ; i <= OPT_MOVIE ; i++ )
        {
          if( options[ i ] )
          {
            groupid = gid_table[ (i - OPT_SYSTEM) ];
          }
        }

        /* Open the libraries */
        if( DataTypesBase = OpenLibrary( "datatypes.library", 45UL ) )
        {
          struct DataType *prevdtn = NULL;
          STRPTR           name    = (STRPTR)options[ OPT_NAME ];
          BPTR             lock    = NULL;

          if( name )
          {
            if( lock = Lock( name, ACCESS_READ ) )
            {
              Printf( ">> file '%s' matches the folowing DataTypes: <<\n", name );
            }
          }

          if( lock || (name == NULL) )
          {
            /* Header */
            Printf( "Description:        |Base Name:          |Type:          |Group:\n" );
            Printf( "--------------------+--------------------+---------------+------------\n" );

            /* Print whole list or determine the DataType of the file */
            while( dtn = ObtainDataType( (ULONG)((name)?(DTST_FILE):(DTST_RAM)),
                                         (APTR)((name)?(lock):(NULL)),
                                         DTA_DataType, prevdtn,
                                         DTA_GroupID,  groupid,
                                         TAG_DONE ) )
            {
              /* Release previous DataType */
              ReleaseDataType( prevdtn );

              dth = dtn -> dtn_Header;

              Printf( "%20.20s|%20.20s|%15.15s|%s\n",
                      (dth -> dth_Name),
                      (dth -> dth_BaseName),
                      GetDTString( (ULONG)((dth -> dth_Flags & DTF_TYPE_MASK) + DTMSG_TYPE_OFFSET) ),
                      GetDTString( (dth -> dth_GroupID) ) );

              if( options[ OPT_TOOLS ] )
              {
                PrintDTToolList( dtn );
              }

              prevdtn = dtn;
            }

            /* Release last datatype */
            ReleaseDataType( prevdtn );
          }

          /* Free lock (if we got one) */
          if( name && lock )
          {
            UnLock( lock );
          }

          if( IoErr() )
          {
            PrintFault( IoErr(), NULL );
          }

          CloseLibrary( DataTypesBase );
        }
        else
        {
          Printf( "Couldn't open datatypes.library V45\n" );
        }

        /* Free the allocated memory after ReadArgs */
        FreeArgs( rdargs );
      }
      else
      {
        /* ReadArgs failed */
        PrintFault( IoErr(), "DTDescrlist" );
      }
    }
}


static
void PrintDTToolList( struct DataType *dtn )
{
    if( dtn )
    {
      struct ToolNode *tn = (struct ToolNode *)(&(dtn -> dtn_ToolList));

      while( tn = FindToolNodeA( (struct List *)tn, NULL ) )
      {
        STRPTR which;

        switch( tn -> tn_Tool . tn_Which )
        {
          case TW_MISC:     which = "misc     "; break;
          case TW_INFO:     which = "info     "; break;
          case TW_BROWSE:   which = "browse   "; break;
          case TW_EDIT:     which = "edit     "; break;
          case TW_PRINT:    which = "print    "; break;
          case TW_MAIL:     which = "mail     "; break;
          default:          which = "undefined"; break;
        }

        Printf( "                    + %s \"%s\"\n", which, (tn -> tn_Tool . tn_Program) );
      }
    }

}


