

/*
**
**  $VER: dtimage.c 1.1 (17.3.97)
**  datatypes.library/Examples/dtimage
**
**  Renders datatypesclass (picture) object into window
**
**  Written 1996/97 by Roland 'Gizzy' Mainz
**
*/

/* amiga includes */
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <dos/rdargs.h>
#include <dos/stdio.h>
#include <graphics/gfx.h>
#include <graphics/displayinfo.h>
#include <intuition/intuition.h>
#include <intuition/icclass.h>
#include <intuition/imageclass.h>
#include <datatypes/datatypes.h>
#include <datatypes/datatypesclass.h>
#include <datatypes/textclass.h>
#include <datatypes/pictureclass.h>
#include <datatypes/soundclass.h>
#include <datatypes/animationclass.h>
#include <workbench/icon.h>

/* amiga prototypes */
#include <clib/macros.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#include <clib/exec_protos.h>
#include <clib/utility_protos.h>
#include <clib/dos_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/datatypes_protos.h>
#include <clib/icon_protos.h>

/* amiga pragmas */
#include <pragmas/exec_pragmas.h>
#include <pragmas/utility_pragmas.h>
#include <pragmas/dos_pragmas.h>
#include <pragmas/graphics_pragmas.h>
#include <pragmas/intuition_pragmas.h>
#include <pragmas/datatypes_pragmas.h>
#include <pragmas/icon_pragmas.h>
#include <pragmas/alib_pragmas.h> /* tagcall stubs */

/* ansi includes */
#include <string.h>
#include <limits.h>

/* macros */
#define IsDTMethodSupported( o, id ) ((BOOL)FindMethod(GetDTMethods( (o) ), (id) ))


/*****************************************************************************/

extern struct Library *SysBase,
                      *DOSBase;

/*****************************************************************************/

struct Library *UtilityBase;
struct Library *GfxBase;
struct Library *IntuitionBase;
struct Library *DataTypesBase;
struct Library *IconBase;

/* prototypes */
static void  PrintErrorMsg( ULONG, STRPTR );

static ULONG DoDTFrameBoxSpecify( Object *, struct Window *, struct Requester *, struct FrameInfo * );
static void  PrintFrameInfo( struct FrameInfo * );

struct FrameInfo gfri = { 0 };

/*****************************************************************************/

/* If condition is false, replace tag with TAG_IGNORE */
#define XTAG( expr, tagid ) ((Tag)((expr)?(tagid):(TAG_IGNORE)))

/*****************************************************************************/

#define TEMPLATE "NAME/A,NOPICONLY/S,DATATYPE/K,PUBSCREEN/K"
#define OPT_NAME         (0)
#define OPT_NOPICONLY    (1)
#define OPT_DATATYPE     (2)
#define OPT_PUBSCREEN    (3)
#define NUM_OPTS         (4)

/*****************************************************************************/

#define IDCMP_FLAGS (IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY | IDCMP_RAWKEY)

/*****************************************************************************/

static
void PrintErrorMsg( ULONG errnum, STRPTR name )
{
    STRPTR format;
    TEXT   errbuff[ 120 ];

    if( errnum >= DTERROR_UNKNOWN_DATATYPE )
    {
      format = GetDTString( errnum );

      if( (ULONG)(strlen( format ) + strlen( name )) < 110UL )
      {
        sprintf( errbuff, format, name );
      }
      else
      {
        /* buffer overflow */
        errbuff[ 0 ] = 0U;
      }
    }
    else
    {
      Fault( errnum, NULL, errbuff, sizeof( errbuff ) );
    }

    Printf( "%s\nerror #%ld\n", errbuff, errnum );
}

/*****************************************************************************/

void main( void )
{
    Object               *dto;

    ULONG                 modeid    = (ULONG)INVALID_ID;
    BOOL                  useScreen = FALSE;
    struct Screen        *scr;
    struct Window        *win;
    BOOL                  going     = TRUE;

    struct IntuiMessage  *imsg;
    ULONG                 sigmask,
                          signals;

    LONG                  options[ NUM_OPTS ];
    struct RDArgs        *rdargs;

    /* Parse the arguments.  Note that this simple example assumes
     * that it was started from the shell.
     */
    memset( options, 0, sizeof( options ) );

    if( rdargs = ReadArgs( TEMPLATE, options, NULL ) )
    {
      /* Open the other libraries */
      UtilityBase   = OpenLibrary( "utility.library",   39UL );
      GfxBase       = OpenLibrary( "graphics.library",  39UL );
      IntuitionBase = OpenLibrary( "intuition.library", 39UL );
      DataTypesBase = OpenLibrary( "datatypes.library", 45UL );
      IconBase      = OpenLibrary( "icon.library",      39UL );

      if( UtilityBase && GfxBase && IntuitionBase && DataTypesBase && IconBase )
      {
        struct DataType *dtn = NULL;

        if( options[ OPT_DATATYPE ] )
        {
          if( !(dtn = ObtainDataType( DTST_RAM, (APTR)options[ OPT_DATATYPE ],
                                      XTAG( (!options[ OPT_NOPICONLY ]), DTA_GroupID ), GID_PICTURE,
                                      TAG_DONE )) )
          {
            Printf( "can't find given datatype '%s'\n", (APTR)options[ OPT_DATATYPE ] );
          }
        }

        /* Get a DataType object */
        if( dto = NewDTObject( (APTR)options[ OPT_NAME ], GA_RelVerify, TRUE,
                                                          GA_Immediate, TRUE,
                                                          DTA_DataType, dtn,
                                                          XTAG( (!options[ OPT_NOPICONLY ]), DTA_GroupID ), GID_PICTURE,
                                                          TAG_DONE ) )
        {
          struct dtFrameBox     dtf = { 0 };
          struct FrameInfo      fri = { 0 };
          STRPTR                name;

          /* Get information about the object */
          if( GetDTAttrs( dto,
                          /* Get the name of the object */
                          DTA_ObjName,      (ULONG)(&name),

                          /* Get the mode ID for graphical objects */
                          PDTA_ModeID,      (ULONG)(&modeid),
                          TAG_DONE ) )
          {
            /* Display any information we obtained */
            if( name )
            {
              Printf( "opened \"%s\"\n", name );
            }

            /* Show the mode ID */
            Printf( "mode ID %08lx\n", modeid );
          }

          Printf( "Asking the object what kind of environment it needs...\n" );

          /* Ask the object what kind of environment it needs */
          dtf . MethodID          = DTM_FRAMEBOX;
          dtf . dtf_FrameInfo     = (&fri);
          dtf . dtf_ContentsInfo  = (&fri);
          dtf . dtf_SizeFrameInfo = sizeof( struct FrameInfo );

          if( DoDTMethodA( dto, NULL, NULL, (Msg)(&dtf) ) )
          {
            /* Really success ? */
            if( fri . fri_Dimensions . Depth )
            {
              Printf( "DTM_FRAMEBOX returned\n" );

              PrintFrameInfo( (&fri) );

              if( (fri . fri_PropertyFlags) & DIPF_IS_HAM )
              {
                useScreen = TRUE;
              }

              if( (fri . fri_PropertyFlags) & DIPF_IS_EXTRAHALFBRITE )
              {
                useScreen = TRUE;
              }

              if( (fri . fri_PropertyFlags == 0UL) && (modeid & HAM_KEY) && (modeid != INVALID_ID) )
              {
                useScreen = TRUE;
              }
            }
            else
            {
              Printf( "DTM_FRAMEBOX failed\n" );
            }
          }
          else
          {
            Printf( "DTM_FRAMEBOX failed / couldn't obtain environment information\n" );
          }

          if( useScreen )
          {
            Printf( "this object requires a private screen\n" );
          }

          /* Get a lock on the specified public screen */
          if( scr = LockPubScreen( (STRPTR)options[ OPT_PUBSCREEN ] ) )
          {
            struct IBox       domain;
            ULONG             nomwidth  = 0UL,
                              nomheight = 0UL;

            if( DoDTDomainA( dto, NULL, NULL, NULL, GDOMAIN_NOMINAL, (&domain), NULL ) )
            {
              nomwidth  = domain . Width;
              nomheight = domain . Height;

              Printf( "nom box %d %d %d %d\n", domain );
            }

            /* Make sure we have the right dimensions */
            nomwidth  = ((nomwidth)?(nomwidth):(600UL));
            nomheight = ((nomheight)?(nomheight):(175UL));

            /* Open the window */
            if( win = OpenWindowTags( NULL, WA_InnerWidth,    (nomwidth  + 4UL),
                                            WA_InnerHeight,   (nomheight + 2UL),
                                            WA_Title,         name,
                                            WA_IDCMP,         IDCMP_FLAGS,
                                            WA_DragBar,       TRUE,
                                            WA_DepthGadget,   TRUE,
                                            WA_AutoAdjust,    TRUE,
                                            WA_Activate,      TRUE,
                                            WA_PubScreen,     scr,
                                            TAG_DONE ) )
            {
              APTR drawhandle;

              DoDTFrameBoxSpecify( dto, win, NULL, (&gfri) );

              if( drawhandle = ObtainDTDrawInfo( dto, PDTA_Screen, scr, TAG_DONE ) )
              {
                Printf( "DrawDTObjectA returned %lu\n",
                         DrawDTObjectA( (win -> RPort), dto,
                                        ((win -> BorderLeft) + 2L),
                                        ((win -> BorderTop)  + 1L),
                                        ((win -> Width)  - ((win -> BorderLeft) + (win -> BorderRight)  + 4L)),
                                        ((win -> Height) - ((win -> BorderTop)  + (win -> BorderBottom) + 2L)),
                                        0L, 0L, NULL ) );

                sigmask = ((1UL << win -> UserPort -> mp_SigBit) | SIGBREAKF_CTRL_C);

                /* Keep going until we're told to stop */
                while( going )
                {
                  /* Wait for an event */
                  signals = Wait( sigmask );

                  /* Did we get a break signal */
                  if( signals & SIGBREAKF_CTRL_C )
                  {
                    going = FALSE;
                  }

                  /* Pull Intuition messages */
                  while( imsg = (struct IntuiMessage *)GetMsg( (win -> UserPort) ) )
                  {
                    /* Handle each message */
                    switch( imsg -> Class )
                    {
                      case IDCMP_CLOSEWINDOW:
                          going = FALSE;
                          break;

                      case IDCMP_VANILLAKEY:
                      {
                          switch( imsg -> Code )
                          {
/* Quit application */
                            case 'Q':
                            case 'q':
                            case  27: /* ESC */
                            {
                                Printf( "Quit application\n" );

                                going = FALSE;
                            }
                                break;
                          }
                      }
                          break;
                    }

                    /* Done with the message, so reply to it */
                    ReplyMsg( (&(imsg -> ExecMessage)) );
                  }
                }
                
                ReleaseDTDrawInfo( dto, drawhandle );
              }
              else
              {
                Printf( "ObtainDTDrawInfoA failed / can't obtain draw handle\n" );
              }

              Printf( "Done...\n" );

              /* Close the window now */
              CloseWindow( win );
            }
            else
            {
              Printf( "Couldn't open window\n" );
            }

            /* Unlock the screen */
            UnlockPubScreen( NULL, scr );
          }
          else
          {
            Printf( "Couldn't lock public screen\n" );
          }

          /* Dispose of the DataType object */
          DisposeDTObject( dto );
        }
        else
        {
          PrintErrorMsg( IoErr(), (STRPTR)options[ OPT_NAME ] );
        }

        /* Release datatype */
        ReleaseDataType( dtn );
      }
      else
      {
        Printf( "couldn't open a shared library\n" );
      }

      /* Close the libraries */
      CloseLibrary( IconBase );
      CloseLibrary( DataTypesBase );
      CloseLibrary( IntuitionBase );
      CloseLibrary( GfxBase );
      CloseLibrary( UtilityBase );

      FreeArgs( rdargs );
    }
    else
    {
      /* ReadArgs failed */
      PrintErrorMsg( IoErr(), "DTImage" );
    }
}


/* Tell the object about the environment it will be going into */
static
ULONG DoDTFrameBoxSpecify( Object *dto, struct Window *win, struct Requester *req, struct FrameInfo *fri )
{
    if( dto && fri )
    {
      struct Screen       *scr;
      struct DisplayInfo   di;
      ULONG                modeid;
      struct dtFrameBox    dtf;

      scr = win -> WScreen;

      /* Get the display information */
      modeid = GetVPModeID( (&(scr -> ViewPort)) );
      (void)GetDisplayInfoData( NULL, (APTR)(&di), sizeof( struct DisplayInfo ), DTAG_DISP, modeid );

      /* Fill in the frame info */
      memset( (void *)fri, 0, sizeof( struct FrameInfo ) );

      fri -> fri_PropertyFlags       = di . PropertyFlags;
      fri -> fri_Resolution          = *(&(di . Resolution));
      fri -> fri_RedBits             = di . RedBits;
      fri -> fri_GreenBits           = di . GreenBits;
      fri -> fri_BlueBits            = di . BlueBits;
      fri -> fri_Dimensions . Width  = scr -> Width;
      fri -> fri_Dimensions . Height = scr -> Height;
      fri -> fri_Dimensions . Depth  = scr -> BitMap . Depth;
      fri -> fri_Screen              = scr;
      fri -> fri_ColorMap            = scr -> ViewPort . ColorMap;

      /* Send the message */
      dtf . MethodID          = DTM_FRAMEBOX;
      dtf . dtf_GInfo         = NULL;
      dtf . dtf_ContentsInfo  = fri;
      dtf . dtf_FrameInfo     = NULL;
      dtf . dtf_SizeFrameInfo = sizeof(struct FrameInfo);
      dtf . dtf_FrameFlags    = FRAMEF_SPECIFY;

      return( DoDTMethodA( dto, win, req, (Msg)(&dtf) ) );
    }

    return( 0UL );
}


static
void PrintFrameInfo( struct FrameInfo *fri )
{
    Printf( "fri: PropertyFlags : 0x%lx\n", (ULONG)(fri -> fri_PropertyFlags) );
    Printf( "fri: RedBits       : 0x%lx\n", (ULONG)(fri -> fri_RedBits) );
    Printf( "fri: GreenBits     : 0x%lx\n", (ULONG)(fri -> fri_GreenBits) );
    Printf( "fri: BlueBits      : 0x%lx\n", (ULONG)(fri -> fri_BlueBits) );
    Printf( "fri: Width         : %lu\n",   (ULONG)(fri -> fri_Dimensions . Width) );
    Printf( "fri: Height        : %lu\n",   (ULONG)(fri -> fri_Dimensions . Height) );
    Printf( "fri: Depth         : %lu\n",   (ULONG)(fri -> fri_Dimensions . Depth) );
    Printf( "fri: Screen        : 0x%lx\n", (ULONG)(fri -> fri_Screen) );
    Printf( "fri: ColorMap      : 0x%lx\n", (ULONG)(fri -> fri_ColorMap) );

    if( (fri -> fri_PropertyFlags) & DIPF_IS_HAM )
    {
      Printf( "fri: HAM\n" );
    }

    if( (fri -> fri_PropertyFlags) & DIPF_IS_EXTRAHALFBRITE )
    {
      Printf( "fri: ExtraHalfBrite\n" );
    }

    if( (fri -> fri_Flags) & FIF_SCALABLE )
    {
      Printf( "fri: FIF_SCALABLE\n" );
    }

    if( (fri -> fri_Flags) & FIF_SCROLLABLE )
    {
      Printf( "fri: FIF_SCROLLABLE\n" );
    }

    if( (fri -> fri_Flags) & FIF_REMAPPABLE )
    {
      Printf( "fri: FIF_REMAPPABLE\n" );
    }
}




